"""Basic setup file to allow editable pip installs while using the declarative configuration (setup.cfg)."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from setuptools import setup

# This is required
setup()
